package me.lensfrex.manager.utils;

import me.lensfrex.manager.Main;
import me.lensfrex.manager.data.Student;
import me.lensfrex.manager.data.manager.StudentManager;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;

public class Worker {

    public static final String helpMessage = IOUtil.inputStreamToString(Main.class.getResourceAsStream("/me.lensfrex.manager/helpMessage.txt"), StandardCharsets.UTF_8);
    public static final String studentInformationExample = IOUtil.inputStreamToString(Main.class.getResourceAsStream("/me.lensfrex.manager/studentInformationExample.txt"), StandardCharsets.UTF_8);

    private final StudentManager studentManager = new StudentManager();

    public Worker() {
        printHelp();
        if (!studentManager.hasStudent()) {
            System.out.println("It's seems that there's no any student. Please add at least one student to let the program work properly.");
        }
    }

    /**
     * 打印操作信息
     */
    public void printHelp() {
        System.out.println(helpMessage);
    }

    /**
     * 根据用户输入添加学生
     *
     * @param userInput 输入的学生信息，格式参见 @studentInformationExample
     */
    public void addStudent(String userInput) {
        studentManager.addStudent(userInput);
        System.out.println("[System]: Done. (Maybe?)");
    }

    /**
     * 按名字删除某个学生
     *
     * @param name 学生名字
     */
    public void removeStudent(String name) {
        studentManager.removeStudent(name);
        System.out.println("[System]: Done. (Maybe?)");
    }

    /**
     * 按学号删除某个学生
     *
     * @param id 学生学号
     */
    public void removeStudent(long id) {
        studentManager.removeStudent(id);
        System.out.println("[System]: Done. (Maybe?)");
    }

    /**
     * 输出每门课程的总成绩及平均分
     */
    public void printAllSubjectInfo() {
        ArrayList<String> table = studentManager.generateAllSubjectInfo();
        for (String record : table) {
            System.out.println(record);
        }
    }

    /**
     * "输出每个学生的学号、姓名、各科考试成绩、总分、平均分，以及每门课程的总分和平均分"
     */
    public void listRecord() {
        System.out.println(studentManager.generateAllInformation());
    }

    /**
     * 计算每个学生的总分
     */
    public void printAllStudentScoreInfo() {
        ArrayList<Student> students = studentManager.getStudents();

        System.out.println("- All student general score:");
        StringBuilder stringBuilder = new StringBuilder();
        for (Student student : students) {
            stringBuilder
                    .append(student.getName()).append(": ")
                    .append(studentManager.getStudentGeneralScore(student)).append('\n');
        }

        System.out.println(stringBuilder);
    }

    /**
     * 按每个学生的总分由高到低排出名次表
     */
    public void printRankingTable() {
        System.out.println("----------------------------------");
        System.out.println("Ranking  |  Name  |  General Score");
        System.out.println("----------------------------------");

        ArrayList<String> table = studentManager.generateRankingTable();
        for (String record : table) {
            System.out.println(record);
        }
    }

    /**
     * 按学号由小到大排出成绩表
     */
    public void printScoreTableById() {
        System.out.println("---------------------------------------------------------------------");
        System.out.println("ID  |  Name  |  General Score  |  Subject1: score Subject2: score ...");
        System.out.println("---------------------------------------------------------------------");

        ArrayList<String> table = studentManager.generateScoreTableById();
        for (String record : table) {
            System.out.println(record);
        }
    }

    /**
     * 按姓名的字典顺序排出成绩表
     */
    public void printScoreTableByName() {
        System.out.println("---------------------------------------------------------------------");
        System.out.println("ID  |  Name  |  General Score  |  Subject1: score Subject2: score ...");
        System.out.println("---------------------------------------------------------------------");

        ArrayList<String> table = studentManager.generateScoreTableByName();
        for (String record : table) {
            System.out.println(record);
        }
    }

    /**
     * 通过学号查找学生
     */
    public String searchById(long id) {
        return studentManager.getStudentInfo(studentManager.getStudent(id));
    }

    /**
     * 通过名字查找学生
     */
    public String searchByName(String name) {
        return studentManager.getStudentInfo(studentManager.getStudent(name));
    }
}
